close all;
clear all;

%% Estimation of blurring in a a video frame.

% Specify which image to be tested.
image_name = 'lena';
noise_type = 'gaussian';

% Instantiate the metric result vectors. 
% 10 tests are performed.
metric_red = zeros(10,1);
metric_green = zeros(10,1);
metric_blue = zeros(10,1);
psnr_value_red_first = zeros(10,1);

% Load image.
original_image = imread([image_name '.bmp']);

% Metric index instantiation.
index = 1;

% Loop for testing with different noise intensities.
for noise_intensity = 0.01:0.01:0.1
    
    disp = ['Calculating metric for noise variance ' num2str(noise_intensity) '.'];    
    display(disp);
    
    % Read in two generated frames with the specified distortions.
    
    image_string_frame = 'lake.bmp'%[image_name '_' noise_type '_noise_with_variance_' num2str(noise_intensity) '_firstFrame.bmp'];

    frame = imread(image_string_frame);
    
    [rows, cols, dims] = size(frame);
    
    % Separate the channels:
    red = frame(:,:,1);
    green = frame(:,:,2);
    blue = frame(:,:,3);
    
    % Metric calculations.
    % Gaussian operator:
    template = [1/16 2/16 1/16; 2/16 4/16 2/16; 1/16 2/16 1/16];
    
    % Filter the image through the Gaussian filter.
    filtered_frame_red = conv2(double(red), double(template));
    filtered_frame_green = conv2(double(green), double(template));
    filtered_frame_blue = conv2(double(blue), double(template));
    
    filtered_frame_red = uint8(filtered_frame_red(2:rows+1, 2:cols+1));
    filtered_frame_green = uint8(filtered_frame_green(2:rows+1, 2:cols+1));
    filtered_frame_blue = uint8(filtered_frame_blue(2:rows+1, 2:cols+1));
    
    % Find the differences:
    diff_red = abs(double(frame(:,:,1)) - double(filtered_frame_red));
    diff_green = abs(double(frame(:,:,2)) - double(filtered_frame_green));
    diff_blue = abs(double(frame(:,:,3)) - double(filtered_frame_blue));
    
    % Find the SAD.
    for i = 1:rows
        for j = 1:cols
            metric_red(index) = metric_red(index) + int32(diff_red(i,j));
            metric_green(index) = metric_green(index) + int32(diff_green(i,j));
            metric_blue(index) = metric_blue(index) + int32(diff_blue(i,j));
        end
    end
    
    % Calculate the PSNR for comparrison.
    %psnr_value_red_first(index) = PSNR(original_image(:,:,1),frame(:,:,1));
    
    % Update the metric vector index.
    index = index + 1;
end

% Save metric data to a text file as a latex table.

    % Noise variance. 
    nv = 0.01:0.01:0.1;
    nv = nv';

    % Metric data as a matrix.
    data = [nv metric_red metric_green metric_blue];
    
    % Call to function for saving data to latex format in file.
    %saveDataToLatex(data, 'noise_estimation_001_01', 'txt');