close all;
clear all;

%% Estimation of blur in a image with increasing loss of sharpness.

% Specify which image to be tested.
image_name = 'blocks';
noise_type = 'blur';

% Instantiate the metric result vectors.
metric_red = zeros(10,1);
metric_green = zeros(10,1);
metric_blue = zeros(10,1);
psnr_value_red = zeros(10,1);

% Load image.
original_image = imread([image_name '.bmp']);

% Metric index instantiation.
index = 1;

% Loop for testing with different distortion intensities.
for noise_intensity = 0.1:0.1:1

    disp = ['Calculating blur metric for distortion intensity ' num2str(noise_intensity) '.'];    
    display(disp);
    
    % Read in a frame generated with the specified distortions.
    image_string_frame = 'blocks.bmp'%[image_name '_' noise_type '_noise_with_intensity_' num2str(noise_intensity) '_firstFrame.bmp'];
    frame = imread(image_string_frame);
    [rows, cols, dims] = size(frame);
    
    % Find the level of blurriness/sharpness by using a Laplace filter.
    
    % Laplace operator:
    template = [0 1 0; 1 -4 1; 0 1 0];
    
    % Filter the image through the Laplace filter
    filtered_frame_red = conv2(double(frame(:,:,1)), double(template));
    filtered_frame_green = conv2(double(frame(:,:,2)), double(template));
    filtered_frame_blue = conv2(double(frame(:,:,3)), double(template));
    
    % Update the metric result vector.
    metric_red(index) = max(max(abs(filtered_frame_red)));
    metric_green(index) = max(max(abs(filtered_frame_green)));
    metric_blue(index) = max(max(abs(filtered_frame_blue)));
    
    % Calculate the PSNR for comparison.
    %psnr_value_red(index) = PSNR(original_image(:,:,1),frame(:,:,1));
    
    % Update the metric index.
    index = index + 1;
    
end

% Save metric data to a text file as a latex table.

    % Noise variance. 
    nv = 0.01:0.01:0.1;
    nv = nv';

    % Metric data as a matrix.
    data = [nv metric_red metric_green metric_blue];
    
    % Call to function for saving data to latex format in file.
    %saveDataToLatex(data, 'data_to_latex', 'txt');