close all;
clear all;

%% Estimation of random noise in a video sequence (tests over two frames) using A&D metric.

% Specify which image to be tested.
image_name = 'lena';
noise_type = 'blur';

% Instantiate the metric result vectors. 
% 10 tests are performed.
metric_red = zeros(10,1);
metric_green = zeros(10,1);
metric_blue = zeros(10,1);
psnr_value_red_first = zeros(10,1);
psnr_value_red_second = zeros(10,1);

% Load image.
original_image = imread([image_name '.bmp']);

% Metric index instantiation.
index = 1;

% Loop for testing with different noise intensities.
for noise_intensity = 0.1:0.1:0.1
    
    disp = ['Calculating metric for noise variance ' num2str(noise_intensity) '.'];    
    display(disp);
    
    % Read in two generated frames with the specified distortions.
    
    image_string_firstFrame = [image_name '_' noise_type '_noise_with_intensity_' num2str(noise_intensity) '_firstFrame.bmp'];
    image_string_secondFrame = [image_name '_' noise_type '_noise_with_intensity_' num2str(noise_intensity) '_secondFrame.bmp'];

    noisy_first = imread(image_string_firstFrame);
    noisy_second = imread(image_string_firstFrame);
    
    [rows, cols, dims] = size(noisy_first);
    
    % Metric calculations.
    % Instantiate the previous row value registers.
    red_row_registers = zeros(rows,1);
    green_row_registers = zeros(rows,1);
    blue_row_registers = zeros(rows,1);

    % Instantiate the current row accumulation registers.
    red_row_acc_registers = int32(0);
    green_row_acc_registers = int32(0);
    blue_row_acc_registers = int32(0);
    
    % Loop for accessing each pixel for first frame calculations.
    for i = 1:rows
        for j = 1:cols
            % Accumulate the values in the current row.
            red_row_acc_registers = red_row_acc_registers + int32(noisy_first(i,j,1));
            green_row_acc_registers = green_row_acc_registers + int32(noisy_first(i,j,2));
            blue_row_acc_registers = blue_row_acc_registers + int32(noisy_first(i,j,3));
        end
        
        % Store the accumulated values in the previous row value register.
        red_row_registers(i) = red_row_acc_registers;
        green_row_registers(i) = green_row_acc_registers;
        blue_row_registers(i) = blue_row_acc_registers;
        
        % Reset the registers.
        red_row_acc_registers = 0;
        green_row_acc_registers = 0;
        blue_row_acc_registers = 0;
    end

    % Loop for accessing each pixel for second frame calculations.
    for k = 1:rows
        for l = 1:cols
            % Accumulate the values in the current row.
            red_row_acc_registers = red_row_acc_registers + int32(noisy_second(k,l,1));
            green_row_acc_registers = green_row_acc_registers + int32(noisy_second(k,l,2));
            blue_row_acc_registers = blue_row_acc_registers + int32(noisy_second(k,l,3));
        end
        
        % Store the absolute difference between the accumulated values of the first
        % frame and the second frame.
        metric_red(index) = metric_red(index) + abs(red_row_acc_registers - red_row_registers(k));
        metric_green(index) = metric_green(index) + abs(green_row_acc_registers - green_row_registers(k));
        metric_blue(index) = metric_blue(index) + abs(blue_row_acc_registers - blue_row_registers(k));
        
        % Reset the registers.
        red_row_acc_registers = 0;
        green_row_acc_registers = 0;
        blue_row_acc_registers = 0;
    end
    
    % Calculate the PSNR for comparrison.
    psnr_value_red_first(index) = PSNR(original_image(:,:,1),noisy_first(:,:,1));
    psnr_value_red_second(index) = PSNR(original_image(:,:,1),noisy_second(:,:,1));
    
    % Update the metric vector index.
    index = index + 1;
end

% Save metric data to a text file as a latex table.

    % Noise variance. 
    nv = 1:10;
    nv = nv';

    % Metric data as a matrix.
    data = [nv metric_red metric_green metric_blue];
    
    % Call to function for saving data to latex format in file.
    saveDataToLatex(data, 'noise_estimation_001_01', 'txt');