close all;
clear all;

%% Estimation of random noise in a video sequence using the median filter-based metric.

% Specify which image to be tested.
image_name = 'sky';
noise_type = 'blur';

% Instantiate the metric result vectors. 
metric_red = zeros(10,1);
metric_green = zeros(10,1);
metric_blue = zeros(10,1);
psnr_value_red = zeros(10,1);

% Load image
original_image = imread([image_name '.bmp']);
[rows, cols, dims] = size(original_image);

% Metric index instantiation.
index = 1;

% Loop for testing with different noise intensities.
for noise_intensity = 0.1:0.1:1

    disp = ['Calculating median metric for noise intensity ' num2str(noise_intensity) '.'];    
    display(disp);
    
    % Read in generated frame with the specified distortions.
    image_string = [image_name '_' noise_type '_noise_with_intensity_' num2str(noise_intensity) '_firstFrame.bmp'];
    
    noisy_image = imread(image_string);
    
    [rows, cols, dims] = size(noisy_image);
    
    % Separate the channels:
    red = noisy_image(:,:,1);
    green = noisy_image(:,:,2);
    blue = noisy_image(:,:,3);

    % Filter each channel through a 2D median filter:
    filtered_red = medfilt2(red);
    filtered_green = medfilt2(green);
    filtered_blue = medfilt2(blue);

    % Find the absolute differences:
    diff_red = abs(double(red)-double(filtered_red));
    diff_green = abs(double(green)-double(filtered_green));
    diff_blue = abs(double(blue)-double(filtered_blue));

    % Find the sum of absolute differences:
    for i = 1:rows
        for j = 1:cols
            metric_red(index) = metric_red(index) + int32(diff_red(i,j));
            metric_green(index) = metric_green(index) + int32(diff_green(i,j));
            metric_blue(index) = metric_blue(index) + int32(diff_blue(i,j));
        end
    end

    % Calculate the PSNR for comparison.
    psnr_value_red(index) = PSNR(original_image(:,:,1),noisy_image(:,:,1));
    
    % Update metric vector index.
    index = index + 1;
    
end

% Save metric data to a text file as a latex table.

    % Noise variance. 
    nv = 0.1:0.1:1;
    nv = nv';

    % Metric data as a matrix.
    data = [nv metric_red metric_green metric_blue];
    
    % Call to function for saving data to latex format in file.
    saveDataToLatex(data, 'noise_estimation_001_01', 'txt');