--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik Trud Nordeng
-------------------------------------------------------------------------------
-- Description: Shift register utilizing the altera.mf component ALTSHIFT_TAPS.
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

library altera_mf;
use altera_mf.altera_mf_components.all;

entity shift_register is
	generic(							
		data_width : integer := 16;
		size          : integer := 1200
		);
	port(
		clk : in std_logic;
		reset : in std_logic;  
		data_enable : in std_logic;
		data_in : in integer range 0 to 2**data_width - 1;
		data_out : out integer range 0 to 2**data_width - 1
		);
end shift_register;	   

architecture behaviour of shift_register is

COMPONENT altshift_taps
GENERIC (
	intended_device_family		: STRING;
	lpm_hint		: STRING;
	lpm_type		: STRING;
	number_of_taps		: NATURAL;
	tap_distance		: NATURAL;
	width		: NATURAL
);
PORT (
		aclr	: IN STD_LOGIC ;
		clock	: IN STD_LOGIC ;
		shiftout	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		taps	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		clken	: IN STD_LOGIC ;
		shiftin	: IN STD_LOGIC_VECTOR (15 DOWNTO 0)
);
END COMPONENT;

signal data_to_ram : std_logic_vector(15 downto 0);
signal data_from_ram : std_logic_vector(15 downto 0); 
signal shiftout : STD_LOGIC_VECTOR (15 DOWNTO 0);

begin			   
	
	data_to_ram <= (std_logic_vector(to_unsigned(data_in, 16)));
 
	ALTSHIFT_TAPS_component : ALTSHIFT_TAPS
	GENERIC MAP (
		intended_device_family => "Cyclone II",
		lpm_hint => "RAM_BLOCK_TYPE=M4K",
		lpm_type => "altshift_taps",
		number_of_taps => 1,
		tap_distance => 1200,
		width => 16
	)
	PORT MAP (
		aclr => reset,
		clock => clk,
		clken => data_enable,
		shiftin => data_to_ram,
		shiftout => shiftout,
		taps => data_from_ram
	);
	
	data_out <= to_integer(unsigned(data_from_ram));
	
end architecture behaviour;
