--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik Trud Nordeng
-------------------------------------------------------------------------------
-- Description: Top level module for binding together the metric systems for
-- the different color channels.
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity vqa_noiseMetric_ad is
	generic(
		channel_width : integer := 8;
		rows          : integer := 1200;
		cols          : integer := 1920
		);
	port (
		clk          : in  std_logic;
		reset       : in  std_logic;
		
		vsync : in std_logic;
		hsync : in std_logic;
		
		red   : in  integer range 0 to (2**channel_width - 1);
		green : in  integer range 0 to (2**channel_width - 1);
		blue  : in  integer range 0 to (2**channel_width - 1);
		
		metric_red   : out integer range 0 to 1020;
		metric_green : out integer range 0 to 1020;
		metric_blue  : out integer range 0 to 1020
		);
end vqa_noiseMetric_ad;

architecture behaviour of vqa_noiseMetric_ad is	 

	-- Component declaration of the "ad(behaviour)" unit defined in
	-- file: "./../src/accumulate_and_diffferentiate.vhd"
	component ad
	generic(
		channel_width : INTEGER := 8;
		rows : INTEGER := 1200;
		cols : INTEGER := 1920);
	port(
		clk : in STD_LOGIC;
		reset : in STD_LOGIC;
		vsync : in STD_LOGIC;
		hsync : in STD_LOGIC;
		data_in : in INTEGER range 0 to 2**channel_width-1;
		data_out : out INTEGER range 0 to 2**16-1);
	end component;
	for all: ad use entity work.ad(behaviour);

	
begin

	ad_red_component : ad
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_in => red,
		data_out => metric_red
	);
	
	ad_green_component : ad
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_in => green,
		data_out => metric_green
	);
	
	ad_blue_component : ad
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_in => blue,
		data_out => metric_blue
	);
	
end behaviour;

