library IEEE;					
use IEEE.STD_LOGIC_1164.all;

entity stuckAt_analysis is
	generic (
		channel_width : integer := 8
		);
	port (		
		clk : in std_logic;
		reset : in std_logic;
		data_in : in std_logic_vector (channel_width-1 downto 0);
		
		stuck_at_one : out std_logic_vector (channel_width-1 downto 0);
		stuck_at_zero : out std_logic_vector (channel_width-1 downto 0)
		);
end stuckAt_analysis;				   

architecture behaviour of stuckAt_analysis is 
	constant channel_max_value : integer := 2**channel_width - 1;
	signal zero_reg : std_logic_vector (channel_max_value downto 0);
	signal one_reg : std_logic_vector (channel_max_value downto 0);
begin
	
	set_register_proc : process(clk)
	begin							  
		if (rising_edge(clk)) then
			if (reset = '1') then
				zero_reg <= (others => '0');
				one_reg <= (others => '0');
			else   
				for i in 0 to channel_max_value loop
					if (data_in(i) = '1') then
						one_reg(i) <= '1';
					else
						zero_reg(i) <= '1';
					end if;
				end loop;
			end if;
		end if;
	end process set_register_proc; 
	
	stuck_at_one <= one_reg;
	stuck_at_zero <= zero_reg;
	
end architecture behaviour;