--=============================================================================
-- Project: 
-- Copyright: 
-- Author: 
-- Revision:  
-- Last revised: 
-- Workfile: 
-- Archive: 
-------------------------------------------------------------------------------
-- Description:
-- 
-- 
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Revision History:
--
-- 
-- 
--=============================================================================

-- This package defines a set of unsigned arithmetic, conversion,
-- and comparison functions for STD_LOGIC_VECTOR.
library IEEE;					
use IEEE.STD_LOGIC_1164.all;

entity vqa_colorTesting is
	generic (						 
		channel_width : integer := 8;
		rows          : integer := 1080;
		cols          : integer := 1920
		);
	port (		
		
		clk : in std_logic;
		reset : in std_logic;
		
		index : in integer range 0 to 2**channel_width - 1;
		
		hSync : in std_logic;
		vSync : in std_logic;
		
		red : in integer range 0 to 2**channel_width - 1; 
		green : in integer range 0 to 2**channel_width - 1;
		blue : in integer range 0 to 2**channel_width - 1; 
		
		metric_red : out natural;
		metric_green : out natural;
		metric_blue : out natural
		);
end vqa_colorTesting;	  

architecture behaviour of vqa_colorTesting is

-- Record the amount of high values that passes through the system.
	
-- Single-channel histogram accumulator built from counters.

	-- Component declaration of the "histogram_accumulator(behaviour)" unit defined in
	-- file: "./../src/histogram_accumulator.vhd"
	component histogram_accumulator
	generic(
		channel_width : INTEGER := 8;
		rows : INTEGER := 1080;
		cols : INTEGER := 1920;
		bin_max_value : INTEGER := 4096;
		number_of_bins : INTEGER := 256);
	port(
		clk : in STD_LOGIC;
		reset : in STD_LOGIC;
		index : in INTEGER range 0 to 2**channel_width-1;
		data_in : in INTEGER range 0 to 2**channel_width-1;
		count : out INTEGER range 0 to bin_max_value-1);
	end component;
	for all: histogram_accumulator use entity work.histogram_accumulator(behaviour); 


begin
	
	red_channel_histogram : histogram_accumulator
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols,
		bin_max_value => 4096,
		number_of_bins => 8
	)
	port map(
		clk => clk,
		reset => reset,
		index => index,
		data_in => red,
		count => metric_red
	);
	
	green_channel_histogram : histogram_accumulator
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols,
		bin_max_value => 4096,
		number_of_bins => 8
	)
	port map(
		clk => clk,
		reset => reset,
		index => index,
		data_in => green,
		count => metric_green
	);	
	
	blue_channel_histogram : histogram_accumulator
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols,
		bin_max_value => 4096,
		number_of_bins => 8
	)
	port map(
		clk => clk,
		reset => reset,
		index => index,
		data_in => blue,
		count => metric_blue
	);	
	
end architecture behaviour;