library IEEE;					
use IEEE.STD_LOGIC_1164.all;

entity counter is
	generic (						 
		channel_width : integer := 8;
		max_value : integer := 4095
		);
	port (		
		
		clk : in std_logic;
		reset : in std_logic; 
		clock_enable : in std_logic;
		
		data_out : out integer range 0 to max_value
		);
end counter;				   

architecture behaviour of counter is

constant channel_max_value : integer := 2**channel_width - 1;

signal count_register : integer range 0 to max_value;
begin
	count_proc : process(clk)
	begin		 
		if (rising_edge(clk)) then
			if (reset = '1') then
				count_register <= 0;
			elsif(clock_enable = '1') then
				if (count_register = max_value) then
					-- Do nothing to prevent overflow.
				else
					count_register <= count_register + 1;
				end if;
			else
				-- Do nothing.
			end if;
		end if;
	end process count_proc;
	
	data_out <= count_register;
	
end behaviour;