--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik T�rud Nordeng
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity gaussian_metric is
	generic(
		channel_width : integer := 8;
		rows          : integer := 1200;
		cols          : integer := 1920
		);
	port (
		clk          : in  std_logic;
		reset       : in  std_logic;
		
		vsync : in std_logic;
		hsync : in std_logic;
		data_enable : in std_logic;
		data_in   : in  integer range 0 to (2**channel_width - 1);
		
		data_out  : out natural
		);
end gaussian_metric;

architecture behaviour of gaussian_metric is	 
	
	constant metric_max_value : integer := 4095;
	--constant number_of_averaging_samples : integer := 8;
	
	-- Component declaration of the "convolution_filter_3x3(begaviour)" unit defined in
	-- file: "./../src/convolution_filter_3x3.vhd"
	component convolution_filter_3x3
		generic(
			rows : INTEGER := 1200;
			cols : INTEGER := 1920;
			channel_width : INTEGER := 8);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			hsync : in STD_LOGIC;
			vsync : in STD_LOGIC; 
			data_enable : in std_logic;
			data_in : in INTEGER range 0 to 2**channel_width-1;
			original_data_out : out INTEGER range 0 to 255;
			data_out : out INTEGER range 0 to 4095);
	end component;
	for all: convolution_filter_3x3 use entity work.convolution_filter_3x3(begaviour);
	
	signal original_data_from_buffer : integer range 0 to 255;	
	signal data_from_convolution : integer range 0 to 4095;	
	signal absolute_difference : natural;
	signal from_pipeline : natural;
	signal accumulator_reg : natural;
		
begin
	
	convolution_component : convolution_filter_3x3
	generic map(
		rows => rows,
		cols => cols,
		channel_width => channel_width
		)
	port map(
		clk => clk,
		reset => reset,
		hsync => hsync,
		vsync => vsync,	   
		data_enable => data_enable,
		data_in => data_in,
		original_data_out => original_data_from_buffer,
		data_out => data_from_convolution
		); 
		
	pipelining_proc : process(clk)
	begin
		if (rising_edge(clk)) then
			if (reset = '1') then 
				from_pipeline <= 0;
			else 
				from_pipeline <= abs(original_data_from_buffer - data_from_convolution);	
				--absolute_difference <= abs(original_data_from_buffer - data_from_convolution);	
			end if;	
		end if;
	end process pipelining_proc;
	
	absolute_difference <= from_pipeline;
	
	accumulate_proc : process(clk) 
	begin				  
		if (rising_edge(clk)) then
			if (reset = '1') then
				accumulator_reg <= 0;
			else												  
				accumulator_reg <= accumulator_reg + absolute_difference;
			end if;
		end if;														  
	end process accumulate_proc;
	
	data_out <= accumulator_reg;
	
end behaviour;

