--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik Trud Nordeng
-------------------------------------------------------------------------------
-- Description: The code describes the architecture of the row buffers used 
-- for filter implementations.
--=============================================================================		  

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

library altera_mf;
use altera_mf.altera_mf_components.all;

entity row_buffers is	
	generic(
		rows : integer := 1080;
		cols : integer := 1920;
		channel_width : integer := 8	
		);
	port(			  
		-- Clock and reset
		clk : in std_logic;
		reset : in std_logic; 
		
		hsync : in std_logic;
		vsync : in std_logic;
		data_enable : in std_logic;
		-- Data input
		data_in : in integer range 0 to 2**channel_width - 1;		
		
		-- Data output
		data_out_1 : out integer range 0 to 2**channel_width - 1;
		data_out_2 : out integer range 0 to 2**channel_width - 1;
		data_out_3 : out integer range 0 to 2**channel_width - 1
		);
end entity row_buffers;

architecture behaviour of row_buffers is  

COMPONENT altshift_taps
GENERIC (
	intended_device_family		: STRING;
	lpm_hint		: STRING;
	lpm_type		: STRING;
	number_of_taps		: NATURAL;
	tap_distance		: NATURAL;
	width		: NATURAL
);
PORT (
		aclr	: IN STD_LOGIC ;
		clock	: IN STD_LOGIC ;
		shiftout	: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		taps	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		clken	: IN STD_LOGIC ;
		shiftin	: IN STD_LOGIC_VECTOR (7 DOWNTO 0)
);
END COMPONENT;

signal clken : std_logic;

signal data_to_ram : std_logic_vector(7 downto 0);
signal data_from_ram : std_logic_vector(15 downto 0);
signal shiftout : std_logic_vector(7 downto 0);
	
begin	 
	
	data_to_ram <= (std_logic_vector(to_unsigned(data_in, channel_width)));

	ALTSHIFT_TAPS_component : ALTSHIFT_TAPS
	GENERIC MAP (
		intended_device_family => "Cyclone II",
		lpm_hint => "RAM_BLOCK_TYPE=M4K",
		lpm_type => "altshift_taps",
		number_of_taps => 2,
		tap_distance => cols,
		width => 8
	)
	PORT MAP (
		aclr => reset,
		clock => clk,
		clken => data_enable,
		shiftin => data_to_ram,
		shiftout => shiftout,
		taps => data_from_ram
	);	  
	
	-- Value passed straight to the window and filter function.
	data_out_1 <= data_in;
	
	-- Values fetched from the buffers.
	data_out_2 <= to_integer(unsigned(data_from_ram(7 downto 0)));
	data_out_3 <= to_integer(unsigned(data_from_ram(15 downto 8)));
	
end architecture behaviour;
