--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik Trud Nordeng
-------------------------------------------------------------------------------
-- Description: 
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity vqa_blurMetric_gaussian is
	generic(
		channel_width : integer := 8;
		rows          : integer := 1200;
		cols          : integer := 1920
		);
	port (
		clk          : in  std_logic;
		reset       : in  std_logic;
		
		vsync : in std_logic;
		hsync : in std_logic;
		
		data_enable : in std_logic;
		
		red   : in  integer range 0 to (2**channel_width - 1);
		green : in  integer range 0 to (2**channel_width - 1);
		blue  : in  integer range 0 to (2**channel_width - 1);
		
		metric_red   : out natural;
		metric_green : out natural;
		metric_blue  : out natural
		);
end vqa_blurMetric_gaussian;

architecture behaviour of vqa_blurMetric_gaussian is	 
	
	constant metric_max_value : integer := 1020;
	constant number_of_samples : integer := 8;	
	
	-- Component declaration of the "gaussian_metric(behaviour)" unit defined in
	-- file: "./../src/gaussian_metric.vhd"
	component gaussian_metric
	generic(
		channel_width : INTEGER := 8;
		rows : INTEGER := 1200;
		cols : INTEGER := 1920);
	port(
		clk : in STD_LOGIC;
		reset : in STD_LOGIC;
		vsync : in STD_LOGIC;
		hsync : in STD_LOGIC;
		data_enable : in std_logic;
		data_in : in INTEGER range 0 to 2**channel_width-1;
		data_out : out NATURAL);
	end component;
	for all: gaussian_metric use entity work.gaussian_metric(behaviour);
	
begin
	
	red_metric_component : gaussian_metric
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_enable => data_enable,
		data_in => red,
		data_out => metric_red
	);
	
	green_metric_component : gaussian_metric
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_enable => data_enable,
		data_in => green,
		data_out => metric_green
	);
	
	blue_metric_component : gaussian_metric
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_enable => data_enable,
		data_in => blue,
		data_out => metric_blue
	);

end behaviour;

