--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik T?rud Nordeng
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

-- Averaging: A number that typifies a set of numbers of which it is a function.

entity averaging is
	generic(
		rows          : integer := 1080;
		cols          : integer := 1920;
		metric_max_value : integer := 65535;
		number_of_samples : integer := 8	 -- Must be a power of 2 in order to do simple division (shift)
		);
	port(
		clk : in std_logic;
		reset : in std_logic;
		enable : in std_logic;
		data_in : in integer range 0 to metric_max_value;
		data_out : out integer range 0 to metric_max_value
		);
end averaging;


architecture behaviour of averaging is
	type shift_register_t is array (0 to number_of_samples - 1) of integer range 0 to metric_max_value;	-- Need one more cell for subtracting the latest value.
	signal shift_register : shift_register_t;
begin
	
	shift_proc : process (clk)
	begin
		if (rising_edge(clk)) then
			if (reset = '1') then 
				shift_register <= (others => 0);
			elsif (enable = '1') then   
				
				for i in (number_of_samples - 1) downto 1 loop
					shift_register(i) <= shift_register(i - 1);
				end loop;
				
				shift_register(0) <= data_in;
				
			else
				-- Do nothing;
			end if;
		end if;
	end process shift_proc;
	
	averaging_proc : process(clk)
		variable added_values : integer range 0 to 262143;
	begin
		if (rising_edge(clk)) then
			if (reset = '1') then
				added_values := 0;
			elsif(enable = '1') then
				added_values := added_values - shift_register(number_of_samples - 1) + data_in;
			else
				-- Do nothing
			end if;
			
			data_out <= added_values / number_of_samples;
			
		end if;
		
	end process averaging_proc;
	
end architecture behaviour;
