--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik Tørud Nordeng
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity filter is	
	generic(	
		-- Channel bit width:
		channel_width : integer := 8;
		
		-- Laplace operator
		-- Filter parameters:
		t_1_1 : integer := 0;
		t_1_2 : integer := 1;
		t_1_3 : integer := 0;
		
		t_2_1 : integer := 1;
		t_2_2 : integer := -4;
		t_2_3 : integer := 1;
		
		t_3_1 : integer := 0;
		t_3_2 : integer := 1;
		t_3_3 : integer := 0 


--		-- Sobel operator (horizonthal/x)
--		-- Filter parameters:
--		t_1_1 : integer := 1;
--		t_1_2 : integer := 0;
--		t_1_3 : integer := -1;
--		
--		t_2_1 : integer := 2;
--		t_2_2 : integer := 0;
--		t_2_3 : integer := -2;
--		
--		t_3_1 : integer := 1;
--		t_3_2 : integer := 0;
--		t_3_3 : integer := -1 
		
--		-- Sobel operator (vertical/y)
--		-- Filter parameters:
--		t_1_1 : integer := 1;
--		t_1_2 : integer := 2;
--		t_1_3 : integer := 1;
--		
--		t_2_1 : integer := 0;
--		t_2_2 : integer := 0;
--		t_2_3 : integer := 0;
--		
--		t_3_1 : integer := -1;
--		t_3_2 : integer := -2;
--		t_3_3 : integer := -1
		);
	port(	
		clk : in std_logic;
		reset : in std_logic;
		
		vsync : in std_logic;
		hsync : in std_logic;
		
		data_1 : in integer range 0 to (2**channel_width - 1);
		data_2 : in integer range 0 to (2**channel_width - 1);
		data_3 : in integer range 0 to (2**channel_width - 1);	 
		
		data_out : out integer range -1020 to 1020
		);
end entity filter;

architecture behaviour of filter is 

signal w_1_1 : integer range 0 to (2**channel_width - 1);
signal w_1_2 : integer range 0 to (2**channel_width - 1);
signal w_1_3 : integer range 0 to (2**channel_width - 1);
signal w_2_1 : integer range 0 to (2**channel_width - 1);
signal w_2_2 : integer range 0 to (2**channel_width - 1);
signal w_2_3 : integer range 0 to (2**channel_width - 1);
signal w_3_1 : integer range 0 to (2**channel_width - 1);
signal w_3_2 : integer range 0 to (2**channel_width - 1);
signal w_3_3 : integer range 0 to (2**channel_width - 1);

signal template_row_1 : integer range -1020 to 1020; 
signal template_row_2 : integer range -1020 to 1020;
signal template_row_3 : integer range -1020 to 1020;

begin	 
	
	filter_window : process(clk, reset)
	type row_array is array (0 to 2) of integer range 0 to (2**channel_width - 1);
	variable row_1 : row_array;
	variable row_2 : row_array;
	variable row_3 : row_array;
	begin				  
		if (reset = '1') then
			-- RESET
		elsif (rising_edge(clk)) then
			if (vsync = '0') and (hsync = '0') then
				-- Shift values	   
				row_1(0) := row_1(1);
				row_1(1) := row_1(2);
				row_1(2) := data_3;	
				
				row_2(0) := row_2(1);
				row_2(1) := row_2(2);
				row_2(2) := data_2;		
				
				row_3(0) := row_3(1);
				row_3(1) := row_3(2);
				row_3(2) := data_1;	
			else
				-- Sync Active Region
			end if;
		end if;
		-- Link variables to signals
		w_1_1 <= row_1(0);
		w_1_2 <= row_1(1);
		w_1_3 <= row_1(2);
		
		w_2_1 <= row_2(0);
		w_2_2 <= row_2(1);
		w_2_3 <= row_2(2);
		
		w_3_1 <= row_3(0);
		w_3_2 <= row_3(1);
		w_3_3 <= row_3(2);
	end process filter_window;
	
	--filter_function : process(w_1_1, w_1_2, w_1_3, w_2_1, w_2_2, w_2_3, w_3_1, w_3_2, w_3_3) 
	multiplication_proc : process(clk) 
--		variable template_row_1 : integer range -1020 to 1020; 
--		variable template_row_2 : integer range -1020 to 1020;
--		variable template_row_3 : integer range -1020 to 1020;
	begin
		
		-- Following additions lowers the maximum clock frequency! 	
		if (rising_edge(clk)) then
		if (reset = '1') then
			template_row_1 <= 0; 
			template_row_2 <= 0;
			template_row_3 <= 0;
		else
			template_row_1 <= (w_1_1 * t_1_1) + (w_1_2 * t_1_2)	+ (w_1_3 * t_1_3); 
			template_row_2 <= (w_2_1 * t_2_1) + (w_2_2 * t_2_2)	+ (w_2_3 * t_2_3);
			template_row_3 <= (w_3_1 * t_3_1) + (w_3_2 * t_3_2)	+ (w_3_3 * t_3_3);
		end if;					 
		end if;

		--data_out <= template_row_1 + template_row_2 + template_row_3;

	end process multiplication_proc;
	
	addition_proc : process(clk)
	begin	
		if (rising_edge(clk)) then
		data_out <= template_row_1 + template_row_2 + template_row_3;
		end if;
	end process addition_proc;	
	
end architecture behaviour;
