--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik T�rud Nordeng
--=============================================================================	

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity sorter_3 is
	generic(
		channel_width : integer := 8
		);
	port (
		clk : in std_logic;
		reset : in std_logic;
		
		upper_data_in   : in  integer range 0 to (2**channel_width - 1);
		middle_data_in   : in  integer range 0 to (2**channel_width - 1);
		lower_data_in   : in  integer range 0 to (2**channel_width - 1);
		
		upper_data_out  : out integer range 0 to (2**channel_width - 1);
		middle_data_out  : out integer range 0 to (2**channel_width - 1);
		lower_data_out  : out integer range 0 to (2**channel_width - 1)
		);
end sorter_3;

architecture behaviour of sorter_3 is
	
	constant channel_max_value : integer := 2**channel_width - 1;
	
	-- Component declaration of the "sorter_2(behaviour)" unit defined in
	-- file: "./../src/sorter_2.vhd"
	component sorter_2
		generic(
			channel_width : INTEGER := 8
			);
		port(
			upper_data_in : in INTEGER range 0 to 2**channel_width-1;
			lower_data_in : in INTEGER range 0 to 2**channel_width-1;
			upper_data_out : out INTEGER range 0 to 2**channel_width-1;
			lower_data_out : out INTEGER range 0 to 2**channel_width-1);
	end component;
	for all: sorter_2 use entity work.sorter_2(behaviour);
	
	signal to_first_sorter_upper : integer range 0 to channel_max_value; 
	signal to_first_sorter_lower : integer range 0 to channel_max_value;
	
	signal to_second_pipeline_upper : integer range 0 to channel_max_value;
	signal to_second_pipeline_middle : integer range 0 to channel_max_value;
	signal to_second_pipeline_lower : integer range 0 to channel_max_value;
	
	signal to_second_sorter_upper : integer range 0 to channel_max_value; 
	signal to_second_sorter_lower : integer range 0 to channel_max_value; 
	
	signal to_third_pipeline_upper : integer range 0 to channel_max_value;
	signal to_third_pipeline_middle : integer range 0 to channel_max_value;
	signal to_third_pipeline_lower : integer range 0 to channel_max_value;
	
	signal to_third_sorter_upper : integer range 0 to channel_max_value;  
	signal to_third_sorter_lower : integer range 0 to channel_max_value; 
	
begin 
	
	first_pipelining_proc : process(clk)
	begin
		if (rising_edge(clk)) then
			if (reset = '1') then
				to_first_sorter_upper <= 0;
				to_first_sorter_lower <= 0;
				to_second_pipeline_lower <= 0;
			else
				to_first_sorter_upper <=upper_data_in;
				to_first_sorter_lower <= middle_data_in;
				to_second_pipeline_lower <= lower_data_in;
			end if;
		end if;
	end process first_pipelining_proc;
	
	first_sorter_component : sorter_2
	generic map(
		channel_width => channel_width
		)
	port map(
		upper_data_in => to_first_sorter_upper,
		lower_data_in => to_first_sorter_lower,
		upper_data_out => to_second_pipeline_upper,
		lower_data_out => to_second_pipeline_middle
		);
		
	second_pipelining_proc : process(clk)
	begin
		if (rising_edge(clk)) then
			if (reset = '1') then
				to_third_pipeline_upper <= 0;
				to_second_sorter_upper <= 0;
				to_second_sorter_lower <= 0;
			else
				to_third_pipeline_upper <= to_second_pipeline_upper;
				to_second_sorter_upper <= to_second_pipeline_middle;
				to_second_sorter_lower <= to_second_pipeline_lower;
			end if;
		end if;
	end process second_pipelining_proc;
	
	second_sorter_component : sorter_2
	generic map(
		channel_width => channel_width
		)
	port map(
		upper_data_in => to_second_sorter_upper,
		lower_data_in => to_second_sorter_lower,
		upper_data_out => to_third_pipeline_middle,
		lower_data_out => to_third_pipeline_lower
		);
	   
	third_pipelining_proc : process(clk)
	begin
		if (rising_edge(clk)) then
			if (reset = '1') then
				to_third_sorter_upper <= 0;
				to_third_sorter_lower <= 0;
				lower_data_out <= 0;
			else
				to_third_sorter_upper <= to_third_pipeline_upper;
				to_third_sorter_lower <= to_third_pipeline_middle;
				lower_data_out <= to_third_pipeline_lower;
			end if;
		end if;
	end process third_pipelining_proc;
		
	third_sorter_component : sorter_2
	generic map(
		channel_width => channel_width
		)
	port map(
		upper_data_in => to_third_sorter_upper,
		lower_data_in => to_third_sorter_lower,
		upper_data_out => upper_data_out,
		lower_data_out => middle_data_out
		);
	
end architecture behaviour;
