--=============================================================================
-- Project: Video quality analysis using an FPGA
-- Author: Eirik T�rud Nordeng
--=============================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity vqa_noiseMetric_median is
	generic(
		channel_width : integer := 8;
		rows          : integer := 1200;
		cols          : integer := 1920
		);
	port (
		clk          : in  std_logic;
		reset       : in  std_logic;
		
		vsync : in std_logic;
		hsync : in std_logic;
		
		red   : in  integer range 0 to (2**channel_width - 1);
		green : in  integer range 0 to (2**channel_width - 1);
		blue  : in  integer range 0 to (2**channel_width - 1);
		
		metric_red   : out integer;
		metric_green : out integer;
		metric_blue  : out integer
		);
end vqa_noiseMetric_median;

architecture behaviour of vqa_noiseMetric_median is
	
	constant channel_max_value : integer := 2**channel_width - 1;
	
	-- Component declaration of the "noise_metric(behaviour)" unit defined in
	-- file: "./../src/noise_metric.vhd"
	component noise_metric
		generic(
			channel_width : INTEGER := 8;
			rows : INTEGER := 1080;
			cols : INTEGER := 1920);
		port(
			clk : in STD_LOGIC;
			reset : in STD_LOGIC;
			vsync : in STD_LOGIC;
			hsync : in STD_LOGIC;
			data_in : in INTEGER range 0 to 2**channel_width-1;
			metric : out INTEGER);
	end component;
	for all: noise_metric use entity work.noise_metric(behaviour);
	
begin
	
	metric_red_component : noise_metric
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_in => red,
		metric => metric_red
	);
	
	metric_green_component : noise_metric
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_in => green,
		metric => metric_green
	);
	
	metric_blue_component : noise_metric
	generic map(
		channel_width => channel_width,
		rows => rows,
		cols => cols
	)
	port map(
		clk => clk,
		reset => reset,
		vsync => vsync,
		hsync => hsync,
		data_in => blue,
		metric => metric_blue
	);
	
	
end architecture behaviour;

